import sys
import time
import importlib
import os
import numpy as np
os.environ["CUDA_VISIBLE_DEVICES"] = "3"

import torch
import numpy as np
import random
import cv2
import PIL.Image as Image
torch.backends.cudnn.enabled = False
env_path = os.path.join(os.path.dirname(__file__), '..')
if env_path not in sys.path:
    sys.path.append(env_path)

from pytracking.evaluation import Tracker


tracker = Tracker('TALGD', 'default')
tracker_name = tracker.name
parameters = tracker.get_parameters()
tracker_module = importlib.import_module('pytracking.tracker.{}'.format(tracker.name))
tracker_class = tracker_module.get_tracker_class()
tracker = tracker_class(parameters)

def overlay_boxes( image, state,image_file,tracker_name):

    state = torch.tensor(state).reshape(-1, 4)
    boxes = state.clone()
    boxes[:, 2:4] = state[:, 0:2] + state[:, 2:4]

    for box in boxes:
        box = box.to(torch.int64)
        top_left, bottom_right = box[:2].tolist(), box[2:].tolist()
        image = cv2.rectangle(
            image, tuple(top_left), tuple(bottom_right), [0, 0, 255], 2
        )

    # save_path = os.path.join("/media/tangjiuqi097/ext",tracker_name)
    # os.makedirs(save_path, exist_ok=True)
    # split_image_file = image_file.split('/')
    # seq_name = split_image_file[-3]
    # im_name = split_image_file[-1]
    # os.makedirs(os.path.join(save_path, seq_name), exist_ok=True)
    # im_path = os.path.join(save_path, seq_name, im_name)
    # cv2.imwrite(im_path, image[:, :, ::-1])


def _build_init_info(box):
    return {'init_bbox': box, 'init_object_ids': [1, ], 'object_ids': [1, ],
            'sequence_object_ids': [1, ]}

dataset_root = '/data/Disk_C/xuefeng_space/benchmark_videos/sequences/'
video_root = '/data/Disk_C/xuefeng_space/benchmark_videos/sequences/'

video_files = sorted(os.listdir(video_root))

# video_files = ['box_room_noocc_9']
# video_files = ['toy_office_occ_1']
video_files = ['two_mugs']

# video_files = ['XMG_outside', 'backpack_blue', 'backpack_robotarm_lab_occ', 'backpack_room_noocc_1',
#                'bag_outside', 'bicycle2_outside', 'bicycle_outside', 'bottle_box',
#                'bottle_room_noocc_1', 'bottle_room_occ_1', 'box1_outside', 'box_darkroom_noocc_1',
#                'box_darkroom_noocc_10', 'box_darkroom_noocc_2', 'box_darkroom_noocc_3', 'box_darkroom_noocc_4']

model_path = os.path.join('/data/Disk_C/xuefeng_space/VOT2021/TALGD/pytracking/tracking_results', 'rgbd-unsupervised')
vis = True
vid = 0
for video_name in video_files:
    path = os.path.join(model_path, video_name)
    if not os.path.exists(path):
        os.mkdir(path)
    result_path = os.path.join(path, '{}_001.txt'.format(video_name))
    scores_path = os.path.join(path, '{}_001_confidence.value'.format(video_name))
    if os.path.isfile(result_path):
        print('({:3d}) Video : {:12s}   Done!'.format(vid, video_name))
    else:
        video_path = os.path.join(video_root, video_name)

        anno_path = video_path + '/groundtruth.txt'
        rgb_path = video_path + '/color/'
        depth_path = video_path + '/depth/'
        rgb_files = [f for f in sorted(os.listdir(rgb_path))]
        depth_files = [f for f in sorted(os.listdir(depth_path))]
        num_frames = len(rgb_files)
        with open(anno_path) as f:
            ground_truth_list = [i.split('\n')[0].split(',') for i in f.readlines()]
            ground_truth = [(float(i[0]), float(i[1]), float(i[2]), float(i[3])) for i in ground_truth_list]

        ground_truth = np.array(ground_truth, dtype='float32')
        gt_box = ground_truth[0, :]

        toc = 0
        pred_bboxes = []
        scores = []
        track_times = []
        frame = 0
        for frame in range(num_frames):

            im_path = os.path.join(rgb_path, rgb_files[frame])
            depth_image_path = os.path.join(depth_path, depth_files[frame])

            image = cv2.imread(im_path)
            # image = cv2.cvtColor(cv2.imread(im_path), cv2.COLOR_BGR2RGB)
            depth_im = np.array(Image.open(depth_image_path))

            image = Image.fromarray(image)
            size = image.size
            max_size = 960
            scale = max(size) / max_size
            new_size = int(round(size[0] / scale)), int(round(size[1] / scale))
            image = image.resize(new_size, Image.BILINEAR)
            image = np.array(image)

            depth_im = Image.fromarray(depth_im)
            depth_im = depth_im.resize(new_size, Image.BILINEAR)
            depth_im = np.array(depth_im)


            tic = cv2.getTickCount()

            if frame == 0:
                selection = gt_box / scale
                tracker.initialize(image=image, info=_build_init_info(selection), depth_im=depth_im)
                pred_bbox = gt_box
                pred_bboxes.append(pred_bbox)
                scores = [1]
            else:
                pred_bbox, confidence = tracker.track(image=image, depth_im=depth_im)
                # pred_bbox = [t * scale for t in pred_bbox]

                pred_bboxes.append(pred_bbox)
                scores.append(confidence)

            toc += cv2.getTickCount() - tic
            if frame == 0:
                cv2.destroyAllWindows()
            if vis and frame > 0:
                pred_bbox = list(map(int, pred_bbox))
                cv2.rectangle(image, (pred_bbox[0], pred_bbox[1]), (pred_bbox[0]+pred_bbox[2], pred_bbox[1]+pred_bbox[3]), (0, 0, 255), 3)
                cv2.putText(image, str(frame), (40, 40), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 255), 2)
                cv2.imshow(video_name, cv2.resize(image, (640, 480)))
                cv2.waitKey(1)

        toc /= cv2.getTickFrequency()
        with open(result_path, 'w') as f:
            for x in pred_bboxes:
                f.write(','.join([str(i) for i in x]) + '\n')

        with open(scores_path, 'w') as f:
            for s in scores:
                f.write(str(s) + '\n')
        print('({:3d}) Video : {:12s} Time: {:5.1f}s Speed: {:3.1f}fps'.format(vid, video_name, toc, num_frames / toc))


