import sys
import time
import importlib
import os
os.environ["CUDA_VISIBLE_DEVICES"] = "2"
from os.path import join
import torch
import numpy as np
import cv2
import PIL.Image as Image
torch.backends.cudnn.enabled = False
env_path = os.path.join(os.path.dirname(__file__), '..')
if env_path not in sys.path:
    sys.path.append(env_path)

from pytracking.evaluation import Tracker
from pytracking.VOT import vot
# from vot import Rectangle, Polygon, Point
work_dir = os.path.abspath('/data/Disk_C/xuefeng_space/VOT2021/workspace_20')
tracker = Tracker('TALGD', 'default')
tracker_name = tracker.name
parameters = tracker.get_parameters()
tracker_module = importlib.import_module('pytracking.tracker.{}'.format(tracker.name))
tracker_class = tracker_module.get_tracker_class()
tracker = tracker_class(parameters)

def overlay_boxes( image, state,image_file,tracker_name):

    state = torch.tensor(state).reshape(-1, 4)
    boxes = state.clone()
    boxes[:, 2:4] = state[:, 0:2] + state[:, 2:4]

    for box in boxes:
        box = box.to(torch.int64)
        top_left, bottom_right = box[:2].tolist(), box[2:].tolist()
        image = cv2.rectangle(
            image, tuple(top_left), tuple(bottom_right), [0, 0, 255], 2
        )

    # save_path = os.path.join("/media/tangjiuqi097/ext",tracker_name)
    # os.makedirs(save_path, exist_ok=True)
    # split_image_file = image_file.split('/')
    # seq_name = split_image_file[-3]
    # im_name = split_image_file[-1]
    # os.makedirs(os.path.join(save_path, seq_name), exist_ok=True)
    # im_path = os.path.join(save_path, seq_name, im_name)
    # cv2.imwrite(im_path, image[:, :, ::-1])


def _build_init_info(box):
    return {'init_bbox': box, 'init_object_ids': [1, ], 'object_ids': [1, ],
            'sequence_object_ids': [1, ]}


handle = vot.VOT("rectangle", 'rgbd')
region = handle.region()
selection = np.array([region[0], region[1], region[2], region[3]])

imagefile = handle.frame()

imagefile_rgb = imagefile[0]
imagefile_d = imagefile[1]

image = cv2.cvtColor(cv2.imread(imagefile_rgb), cv2.COLOR_BGR2RGB)
depth_im = np.array(Image.open(imagefile_d))


image = Image.fromarray(image)
size = image.size
max_size = 960
scale = max(size) / max_size
new_size = int(round(size[0] / scale)), int(round(size[1] / scale))
image = image.resize(new_size, Image.BILINEAR)
image = np.array(image)

depth_im = Image.fromarray(depth_im)
depth_im = depth_im.resize(new_size, Image.BILINEAR)
depth_im = np.array(depth_im)

selection = selection / scale

tracker.initialize(image=image, info=_build_init_info(selection), depth_im=depth_im)

while True:
    imagefile = handle.frame()

    imagefile_rgb = imagefile[0]
    imagefile_d = imagefile[1]

    image = cv2.cvtColor(cv2.imread(imagefile_rgb), cv2.COLOR_BGR2RGB)
    depth_im = np.array(Image.open(imagefile_d))

    image = Image.fromarray(image)
    size = image.size
    max_size = 960
    scale = max(size) / max_size
    new_size = int(round(size[0] / scale)), int(round(size[1] / scale))
    image = image.resize(new_size, Image.BILINEAR)
    image = np.array(image)

    depth_im = Image.fromarray(depth_im)
    depth_im = depth_im.resize(new_size, Image.BILINEAR)
    depth_im = np.array(depth_im)

    start_time = time.time()

    state, confidence = tracker.track(image=image, depth_im=depth_im)
    pred_bbox = [t * scale for t in state]

    selection = vot.Rectangle(pred_bbox[0], pred_bbox[1], pred_bbox[2], pred_bbox[3])
    handle.report(selection, confidence)
    time.sleep(0.01)
